﻿using System;
using System.Windows;
using System.Windows.Shapes;

namespace Infragistics.Controls.Charts
{
    public class DateTimeValueOverlay : Series
    {
        #region Properties
        public CategoryAxisBase Axis
        {
            get { return (CategoryAxisBase)GetValue(AxisProperty); }
            set { SetValue(AxisProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Axis.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty AxisProperty =
            DependencyProperty.Register("Axis",
            typeof(CategoryAxisBase),
            typeof(DateTimeValueOverlay),
            new PropertyMetadata(null, (sender, e) =>
            {
                DateTimeValueOverlay series = (DateTimeValueOverlay)sender;
                series.RaisePropertyChanged("Axis", e.OldValue, e.NewValue);
            }));

        public int Value
        {
            get { return (int)GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Value.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ValueProperty =
            DependencyProperty.Register("Value",
            typeof(int),
            typeof(DateTimeValueOverlay),
            new PropertyMetadata(0, (sender, e) =>
            {
                DateTimeValueOverlay series = (DateTimeValueOverlay)sender;
                series.RaisePropertyChanged("Value", e.OldValue, e.NewValue);
            }));
        #endregion

        public DateTimeValueOverlay()
        {
            this.DefaultStyleKey = typeof(DateTimeValueOverlay);
        }

        protected override void RenderSeriesOverride(bool animate)
        {
            base.RenderSeriesOverride(animate);

            if (this.Viewport.IsEmpty || this.RootCanvas == null || this.Axis == null)
            {
                return;
            }

            this.RootCanvas.Children.Clear();

            ScalerParams param = new ScalerParams(this.SeriesViewer.WindowRect, this.Viewport, Axis.IsInverted);
            double x = this.Axis.GetScaledValue(Value, param);

            Line lineElement = new Line();
            lineElement.X1 = x;
            lineElement.X2 = x;
            lineElement.Y1 = 0;
            lineElement.Y2 = this.RootCanvas.ActualHeight;
            lineElement.Stroke = this.Brush;
            lineElement.StrokeThickness = this.Thickness;
            lineElement.Opacity = this.Opacity;

            this.RootCanvas.Children.Add(lineElement);
        }

        protected override void RenderThumbnail(Rect viewportRect, RenderSurface surface)
        {
            base.RenderThumbnail(viewportRect, surface);

            if (this.Axis == null || this.RootCanvas == null)
                return;

            ScalerParams param = new ScalerParams(new Rect(0, 0, 1, 1), viewportRect, this.Axis.IsInverted);
            double x = this.Axis.GetScaledValue(Value, param);

            Line lineElement = new Line();
            lineElement.X1 = x;
            lineElement.X2 = x;
            lineElement.Y1 = 0;
            lineElement.Y2 = this.RootCanvas.ActualHeight;
            lineElement.Stroke = this.Brush;
            lineElement.StrokeThickness = this.Thickness;
            lineElement.Opacity = this.Opacity;

            surface.Surface.Children.Add(lineElement);
        }

        protected override void ViewportRectChangedOverride(Rect oldViewportRect, Rect newViewportRect)
        {
            base.ViewportRectChangedOverride(oldViewportRect, newViewportRect);
            this.RenderSeries(false);
        }
        protected override void WindowRectChangedOverride(Rect oldWindowRect, Rect newWindowRect)
        {
            base.WindowRectChangedOverride(oldWindowRect, newWindowRect);
            this.RenderSeries(false);
        }
        protected override void PropertyUpdatedOverride(object sender, string propertyName, object oldValue, object newValue)
        {
            base.PropertyUpdatedOverride(sender, propertyName, oldValue, newValue);
            switch (propertyName)
            {
                // renders series if a new the x-axis is assigned
                case "Axis":
                    {
                        if (oldValue != null)
                        {
                            ((Axis)oldValue).DeregisterSeries(this);
                        }

                        if (newValue != null)
                        {
                            ((Axis)newValue).RegisterSeries(this);
                        }

                        if ((Axis != null && !Axis.UpdateRange()) ||
                            (newValue == null && oldValue != null))
                        {
                            RenderSeries(false);
                            this.NotifyThumbnailDataChanged();
                        }
                    }
                    break;

                case "Value":
                    {
                        if (Value != null && Axis != null)
                        {
                            RenderSeries(false);
                            this.NotifyThumbnailDataChanged();
                        }
                    }
                    break;
            }
        }
    }
}